/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.flink.statefun.flink.core.types.remote;

import java.util.Objects;
import org.apache.flink.api.common.ExecutionConfig;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.common.typeutils.TypeSerializer;
import org.apache.flink.statefun.sdk.TypeName;

public final class RemoteValueTypeInfo extends TypeInformation<byte[]> {

  private static final long serialVersionUID = 1L;

  private final TypeName type;

  public RemoteValueTypeInfo(TypeName type) {
    this.type = Objects.requireNonNull(type);
  }

  @Override
  public TypeSerializer<byte[]> createSerializer(ExecutionConfig executionConfig) {
    return new RemoteValueSerializer(type);
  }

  @Override
  public boolean isBasicType() {
    return false;
  }

  @Override
  public boolean isTupleType() {
    return false;
  }

  @Override
  public int getArity() {
    return 0;
  }

  @Override
  public int getTotalFields() {
    return 0;
  }

  @Override
  public Class<byte[]> getTypeClass() {
    return byte[].class;
  }

  @Override
  public boolean isKeyType() {
    return false;
  }

  @Override
  public String toString() {
    return "RemoteValueTypeInfo{" + "type=" + type + '}';
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    RemoteValueTypeInfo that = (RemoteValueTypeInfo) o;
    return type.equals(that.type);
  }

  @Override
  public int hashCode() {
    return Objects.hashCode(type);
  }

  @Override
  public boolean canEqual(Object obj) {
    return obj instanceof RemoteValueTypeInfo;
  }
}
