/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.flink.statefun.sdk.kafka.testutils;

import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeMatcher;

public final class Matchers {

  private Matchers() {}

  public static <K, V> Matcher<Map<K, V>> isMapOfSize(int size) {
    return new TypeSafeMatcher<Map<K, V>>() {
      @Override
      protected boolean matchesSafely(Map<K, V> map) {
        return map.size() == size;
      }

      @Override
      public void describeTo(Description description) {}
    };
  }

  public static Matcher<Properties> hasProperty(String key, String value) {
    return new TypeSafeMatcher<Properties>() {
      @Override
      protected boolean matchesSafely(Properties properties) {
        return Objects.equals(properties.getProperty(key), value);
      }

      @Override
      public void describeTo(Description description) {
        description.appendText("<" + key + "=" + value + ">");
      }
    };
  }
}
